import pkg_resources

from kikyo.analytic import Analytic
from kikyo.datahub import DataHub
from kikyo.objstore import ObjStore
from kikyo.search import Search
from kikyo.settings import Settings


class Kikyo:
    datahub: DataHub
    objstore: ObjStore
    search: Search
    analytic: Analytic

    settings: Settings

    def __init__(self, _settings: dict = None, **kwargs):
        self.settings = Settings(_settings)
        self.settings.merge(kwargs)
        self._init()

    def _init(self):
        self._init_plugins()

    def _init_plugins(self):
        plugins = {
            entry_point.name: entry_point.load()
            for entry_point in pkg_resources.iter_entry_points('kikyo.plugins')
        }

        active_plugins = self.settings.getlist('active_plugins')
        if active_plugins:
            active_plugins = set(active_plugins)
            for name in list(plugins.keys()):
                if name not in active_plugins:
                    del plugins[name]

        for name, plugin in plugins.items():
            if hasattr(plugin, 'configure_kikyo'):
                plugin.configure_kikyo(self)



from abc import ABCMeta, abstractmethod
from typing import Any


class Bucket(metaclass=ABCMeta):
    def __init__(self, name: str):
        self._name = name

    @abstractmethod
    def get_object_link(self, key: str) -> str:
        """
        获取文件的下载链接

        :param key: 文件的名称
        """

    @abstractmethod
    def put_object(self, key: str, data: Any):
        """上传文件

        :param key: 文件的名称
        :param data: 文件数据
        """


class ObjStore(metaclass=ABCMeta):
    """
    提供文件存储服务
    """

    @abstractmethod
    def bucket(self, name: str) -> Bucket:
        """获取bucket

        :param name: bucket名称
        """
